#ifndef POKEPLATINUM_OV5_021D431C_H
#define POKEPLATINUM_OV5_021D431C_H

#include "struct_decls/struct_0203CDB0_decl.h"
#include "struct_decls/struct_020508D4_decl.h"
#include "overlay005/struct_ov5_021D432C_decl.h"
#include "overlay005/struct_ov5_021D4E00_decl.h"

UnkStruct_ov5_021D432C * ov5_021D431C(void);
void ov5_021D432C(UnkStruct_ov5_021D432C * param0);
void ov5_021D4334(const int param0, const int param1, UnkStruct_ov5_021D432C * param2);
BOOL ov5_021D433C(UnkStruct_0203CDB0 * param0, UnkStruct_ov5_021D432C * param1);
BOOL ov5_021D453C(UnkStruct_0203CDB0 * param0, UnkStruct_ov5_021D432C * param1);
BOOL ov5_021D4858(UnkStruct_0203CDB0 * param0, UnkStruct_ov5_021D432C * param1, const u8 param2);
BOOL ov5_021D4A24(UnkStruct_0203CDB0 * param0, UnkStruct_ov5_021D432C * param1, const u8 param2);
void ov5_021D4BF4(UnkStruct_0203CDB0 * param0, const int param1, const int param2, const u8 param3);
void ov5_021D4C88(UnkStruct_0203CDB0 * param0, const u8 param1);
void ov5_021D4CEC(UnkStruct_0203CDB0 * param0, const u8 param1);
void ov5_021D4D48(UnkStruct_0203CDB0 * param0, const u8 param1);
void ov5_021D4D68(UnkStruct_0203CDB0 * param0, const u8 param1);
void ov5_021D4D78(const int param0, const int param1, const int param2, UnkStruct_0203CDB0 * param3);
UnkStruct_ov5_021D4E00 * ov5_021D4E00(void);
BOOL ov5_021D4E10(UnkStruct_020508D4 * param0);
BOOL ov5_021D4F14(UnkStruct_020508D4 * param0);
BOOL ov5_021D4FA0(UnkStruct_020508D4 * param0);
BOOL ov5_021D5020(UnkStruct_020508D4 * param0);
BOOL ov5_021D5150(UnkStruct_020508D4 * param0);

#endif // POKEPLATINUM_OV5_021D431C_H
